import numpy as np
from scipy.signal import convolve

from lib5c.util.counts import propagate_nans
from lib5c.util.mathematics import zero_nans
from lib5c.util.donut import donut_footprint, lower_left_footprint


def apply_filter(obs_matrix, exp_matrix, footprint, min_percent=0.7):
    """
    Computes a corrected expected value by applying a footprint to observed and
    expected matrix.

    Parameters
    ----------
    obs_matrix, exp_matrix : np.ndarray
        The square, symmetric matrices of observed and expected counts,
        respectively.
    footprint : np.ndarray
        The footprint to convolve. Should contain 1's at positions included in
        the footprint and 0's everywhere else.
    min_percent : float
        If the proportion of nan's in the footprint for a pixel is greater than
        this value, the corrected expected at that point will be set to nan.

    Returns
    -------
    np.ndarray
        The corrected expected value.
    """
    # propagate nans
    obs, exp = propagate_nans(obs_matrix, exp_matrix)

    # record where the nans are
    nan = ~np.isfinite(obs).astype(int)

    # wipe nans
    obs = zero_nans(obs)
    exp = zero_nans(exp)

    # convolve
    nan_conv = convolve(nan, footprint, mode='same')
    obs_conv = convolve(obs, footprint, mode='same')
    exp_conv = convolve(exp, footprint, mode='same')

    # compute corrected expected
    result = (obs_conv / exp_conv) * exp_matrix

    # convert min_percent to a number of nans and apply the threshold
    min_n = np.sum(footprint) * min_percent
    result[nan_conv > min_n] = np.nan

    # write back nans
    result[nan == 1] = np.nan

    return result


def donut_filt(obs_matrix, exp_matrix, p, w, min_percent=0.7):
    """
    Computes the full donut filter.

    Parameters
    ----------
    obs_matrix, exp_matrix : np.ndarray
        The square, symmetric matrices of observed and expected counts,
        respectively.
    p, w : int
        The inner and outer radii of the donut, respectively.
    min_percent : float
        If the proportion of nan's in the footprint for a pixel is greater than
        this value, the corrected expected at that point will be set to nan.

    Returns
    -------
    np.ndarray
        The corrected expected value.
    """
    return apply_filter(obs_matrix, exp_matrix, donut_footprint(p, w),
                        min_percent=min_percent)


def lower_left_filt(obs_matrix, exp_matrix, p, w, min_percent=0.7):
    """
    Computes the lower left donut filter.

    Parameters
    ----------
    obs_matrix, exp_matrix : np.ndarray
        The square, symmetric matrices of observed and expected counts,
        respectively.
    p, w : int
        The inner and outer radii of the donut, respectively.
    min_percent : float
        If the proportion of nan's in the footprint for a pixel is greater than
        this value, the corrected expected at that point will be set to nan.

    Returns
    -------
    np.ndarray
        The corrected expected value.
    """
    return apply_filter(obs_matrix, exp_matrix, lower_left_footprint(p, w),
                        min_percent=min_percent)
